#include <p18cxxx.h>
#include <portb.h>
#pragma config WDT = OFF, OSC = INTIO2, FSCM = OFF, IESO = OFF, LVP = OFF, MCLRE = OFF, STVR = OFF, DEBUG = OFF, WRT0 = OFF, WRT1 = OFF, CP0 = OFF, CP1 = OFF, CPD = OFF, BOR = OFF

#include <adc.h>//include library ADC functions






#define LEDPIN LATAbits.LATA0 //A #define command basically makes the compiler do a find-and-replace, 
							//so wherever I write "LEDPIN" it will replace it with LATAbits.LATA0 before
							//compiling. I like to do this to make programs easier to read. Also, if I change
							//which pin is the LED pin, I only have to change it at one location.

#define ADCMAX 1023 //this is the maximum value the ADC result can have, 2^10 - 1.I know this because it's a 10-bit 
					//ADC, so the highest number it can hold is 1111111111 = 1023.
					//The maximum number will correspond to 5V because I configure it below to use the power supply
					//pin (5V) as its reference maximum voltage. (analog to digital converters need a high and low
					//reference voltage to operate. Then they produce a number which represents where the input
					//is relative to the reference voltages. For example, a 2.5V input will produce a value of
					//about 512, because it's halfway between 0V and 5V.

//Now declare the variables we'll use. C18 needs variables to be declared ahead of time. It gives a syntax
//error if you declare new variables in the middle of the main() function.
int ADCresult;
float voltage;

void main (void)
{
	//*****begin standard config*****
	OSCCON = 0x70;	//set it to use the 8MHz internal clock.
	EnablePullups(); //PORTB pullup resistors enabled. Doesn't really matter for this tutorial.
	//*****end standard config*****

	//****begin project specific config
	ADCON1 = 0b01111101;	//set all ADC-capable pins to NOT be analog inputs, just digital IO, except for AN1 (the leftmost bit is meaningless)
	DDRB=0b11111111;  //all inputs.
	DDRA=0b00000010; //all outputs except RA1, which is AN1
					//the order is RA7 - RA0.
  	//pin direction can also be set individually with commands like: DDRAbits.RA0=0;

	//****end project-specific config***


	//First, use the library OpenADC function to "open" and configure the ADC peripheral. Note that this PIC only has one
	//ADC peripheral, even though there are 7 pins that it can connect to. In other words, we can only do
	//one analog-to-digital conversion at a time.
	OpenADC( ADC_FOSC_32    &  //This was recommended in Table 17-2 of the 18LF1320 datasheet, for clock speeds up to 10.65MHz
	    	 ADC_RIGHT_JUST &  //the result will be in the lowest 10 bits.
	     	 ADC_4_TAD, //The 18F1320 datasheet recommended this setting, if I read the datasheet right.
	    	 ADC_CH1  &  //Use pin AN1 i.e. RA1
	    	 ADC_VREFPLUS_VDD & //Set the reference maximum voltage to be the + power supply voltage (5V) and the
	    	 ADC_VREFMINUS_VSS &//reference minimum to the (-) power supply voltage (0V).
	    	 ADC_INT_OFF, 0b01111101  ); //Don't set an interrupt, and set only AN0 to be an analog pin (this was also done above, at the beginning of main(), but the library function does it here too.)

  	
  	while(1)
  	{
	  	ConvertADC(); //This makes it "capture" the analog voltage and start the analog to digital conversion process.
	  	while(BusyADC() )//BusyADC() is true until the conversion process completes. Need to wait for it to complete or we'll get bad data.
	  	{//do nothing and wait for conversion to happen.
		}
		ADCresult = ReadADC();//When the while loop exits, we know the conversion is done. So get the result from the ADC peripheral.
		voltage = (double)ADCresult *(5.0)/ ADCMAX;//Scale it into a voltage by multiplying by 5V/1023
													//The "(double)" forces it to do floating-point math, i.e. give us decimals,
													//rather than integer math.
		//now we have the voltage and can do whatever we want. Let's light an LED if it's greater than 2.8V
		if(voltage > 2.8)
			LEDPIN = 1;  //note that I defined LEDPIN above
		else
			LEDPIN = 0;

	}
	
}
